@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2008 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2009 by Paolo Lucente
 */
 
 /*
@@ -476,10 +476,18 @@ int pm_scandir(const char *dir, struct dirent ***namelist,
     if (select == NULL || (select != NULL && (*select)(entry)))
     {
       *namelist=(struct dirent **)realloc((void *)(*namelist), (size_t)((i+1)*sizeof(struct dirent *)));
-      if (*namelist == NULL) return(-1);
+      if (*namelist == NULL)
+      {
+         closedir(d);
+         return(-1);
+      }
       entrysize=sizeof(struct dirent)-sizeof(entry->d_name)+strlen(entry->d_name)+1;
       (*namelist)[i]=(struct dirent *)malloc(entrysize);
-      if ((*namelist)[i] == NULL) return(-1);
+      if ((*namelist)[i] == NULL)
+      {
+        closedir(d);
+        return(-1);
+      }
       memcpy((*namelist)[i], entry, entrysize);
       i++;
     }
