@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2009 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2010 by Paolo Lucente
 */
 
 /*
@@ -37,12 +37,14 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   struct pollfd pfd;
   struct insert_data idata;
   struct timezone tz;
-  time_t now, refresh_deadline;
+  time_t refresh_deadline;
   int timeout;
   int ret, num;
   struct ring *rg = &((struct channels_list_entry *)ptr)->rg;
   struct ch_status *status = ((struct channels_list_entry *)ptr)->status;
   u_int32_t bufsz = ((struct channels_list_entry *)ptr)->bufsize;
+  struct pkt_bgp_primitives *pbgp;
+  char *dataptr;
 
   unsigned char *rgptr;
   int pollagain = TRUE;
@@ -63,15 +65,14 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
   /* some LOCAL initialization AFTER setting some default values */
   reload_map = FALSE;
-  timeout = config.sql_refresh_time*1000; /* dirty */
-  now = time(NULL);
-  refresh_deadline = now;
+  idata.now = time(NULL);
+  refresh_deadline = idata.now;
 
   sql_init_maps(&nt, &nc, &pt);
   sql_init_global_buffers();
   sql_init_pipe(&pfd, pipe_fd);
-  sql_init_historical_acct(now, &idata);
-  sql_init_triggers(now, &idata);
+  sql_init_historical_acct(idata.now, &idata);
+  sql_init_triggers(idata.now, &idata);
   sql_init_refresh_deadline(&refresh_deadline);
 
   /* setting number of entries in _protocols structure */
@@ -97,6 +98,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   for(;;) {
     poll_again:
     status->wakeup = TRUE;
+    sql_calc_refresh_timeout(refresh_deadline, idata.now, &timeout);
     ret = poll(&pfd, 1, timeout);
     if (ret < 0) goto poll_again;
 
@@ -104,7 +106,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
     switch (ret) {
     case 0: /* timeout */
-      if (qq_ptr) sql_cache_flush(queries_queue, qq_ptr, &idata);
+      if (qq_ptr) sql_cache_flush(queries_queue, qq_ptr, &idata, FALSE);
       switch (fork()) {
       case 0: /* Child */
 	/* we have to ignore signals to avoid loops:
@@ -126,9 +128,11 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	  
         exit(0);
       default: /* Parent */
+	if (pqq_ptr) sql_cache_flush_pending(pending_queries_queue, pqq_ptr, &idata);
 	gettimeofday(&idata.flushtime, &tz);
-	refresh_deadline += config.sql_refresh_time; 
-	if (idata.now > idata.triggertime) {
+	while (idata.now > refresh_deadline)
+	  refresh_deadline += config.sql_refresh_time; 
+	while (idata.now > idata.triggertime && idata.t_timeslot > 0) {
 	  idata.triggertime  += idata.t_timeslot;
 	  if (config.sql_trigger_time == COUNT_MONTHLY)
 	    idata.t_timeslot = calc_monthly_timeslot(idata.triggertime, config.sql_trigger_time_howmany, ADD);
@@ -136,7 +140,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	idata.new_basetime = FALSE;
 	glob_new_basetime = FALSE;
 	qq_ptr = pqq_ptr;
-	memcpy(queries_queue, pending_queries_queue, sizeof(queries_queue));
+	memcpy(queries_queue, pending_queries_queue, qq_ptr*sizeof(struct db_cache *));
 
 	if (reload_map) {
 	  load_networks(config.networks_file, &nt, &nc);
@@ -184,7 +188,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
       /* lazy sql refresh handling */ 
       if (idata.now > refresh_deadline) {
-        if (qq_ptr) sql_cache_flush(queries_queue, qq_ptr, &idata);
+        if (qq_ptr) sql_cache_flush(queries_queue, qq_ptr, &idata, FALSE);
         switch (fork()) {
         case 0: /* Child */
           /* we have to ignore signals to avoid loops:
@@ -206,9 +210,11 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
           exit(0);
         default: /* Parent */
+	  if (pqq_ptr) sql_cache_flush_pending(pending_queries_queue, pqq_ptr, &idata);
 	  gettimeofday(&idata.flushtime, &tz);
-	  refresh_deadline += config.sql_refresh_time; 
-          if (idata.now > idata.triggertime) {
+	  while (idata.now > refresh_deadline)
+	    refresh_deadline += config.sql_refresh_time; 
+          while (idata.now > idata.triggertime && idata.t_timeslot > 0) {
 	    idata.triggertime  += idata.t_timeslot;
 	    if (config.sql_trigger_time == COUNT_MONTHLY)
 	      idata.t_timeslot = calc_monthly_timeslot(idata.triggertime, config.sql_trigger_time_howmany, ADD);
@@ -216,7 +222,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	  idata.new_basetime = FALSE;
 	  glob_new_basetime = FALSE;
 	  qq_ptr = pqq_ptr;
-	  memcpy(queries_queue, pending_queries_queue, sizeof(queries_queue));
+	  memcpy(queries_queue, pending_queries_queue, qq_ptr*sizeof(struct db_cache *));
 
 	  if (reload_map) {
 	    load_networks(config.networks_file, &nt, &nc);
@@ -228,7 +234,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       } 
       else {
 	if (config.sql_trigger_exec) {
-	  if (idata.now > idata.triggertime) {
+	  while (idata.now > idata.triggertime && idata.t_timeslot > 0) {
 	    sql_trigger_exec(config.sql_trigger_exec); 
 	    idata.triggertime += idata.t_timeslot;
 	    if (config.sql_trigger_time == COUNT_MONTHLY)
@@ -238,13 +244,17 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       }
 
       data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
-      if (idata.now > (idata.basetime + idata.timeslot)) {
-	idata.basetime += idata.timeslot;
-	if (config.sql_history == COUNT_MONTHLY)
-	  idata.timeslot = calc_monthly_timeslot(idata.basetime, config.sql_history_howmany, ADD);
-	glob_basetime = idata.basetime;
-	idata.new_basetime = TRUE;
-	glob_new_basetime = TRUE;
+      if (config.sql_history) {
+        while (idata.now > (idata.basetime + idata.timeslot)) {
+	  time_t saved_basetime = idata.basetime;
+
+	  idata.basetime += idata.timeslot;
+	  if (config.sql_history == COUNT_MONTHLY)
+	    idata.timeslot = calc_monthly_timeslot(idata.basetime, config.sql_history_howmany, ADD);
+	  glob_basetime = idata.basetime;
+	  idata.new_basetime = saved_basetime;
+	  glob_new_basetime = saved_basetime;
+	}
       }
 
       while (((struct ch_buf_hdr *)pipebuf)->num) {
@@ -256,10 +266,17 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	  if (!pt.table[data->primitives.dst_port]) data->primitives.dst_port = 0;
 	}
 
-	(*insert_func)(data, &idata);
+        if (PbgpSz) pbgp = (struct pkt_bgp_primitives *) ((u_char *)data+PdataSz);
+        else pbgp = NULL;
+
+	(*insert_func)(data, pbgp, &idata);
 	
 	((struct ch_buf_hdr *)pipebuf)->num--;
-        if (((struct ch_buf_hdr *)pipebuf)->num) data++;
+        if (((struct ch_buf_hdr *)pipebuf)->num) {
+          dataptr = (unsigned char *) data;
+          dataptr += PdataSz + PbgpSz;
+          data = (struct pkt_data *) dataptr;
+	}
       }
       goto read_data;
     }
@@ -395,6 +412,7 @@ int MY_cache_dbop(struct DBdesc *db, struct db_cache *cache_elem, struct insert_
 
 void MY_cache_purge(struct db_cache *queue[], int index, struct insert_data *idata)
 {
+  struct db_cache *LastElemCommitted = NULL;
   struct logfile lf;
   time_t start;
   int j, stop, ret;
@@ -403,7 +421,7 @@ void MY_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
   memset(&lf, 0, sizeof(struct logfile));
 
   for (j = 0, stop = 0; (!stop) && preprocess_funcs[j]; j++)
-    stop = preprocess_funcs[j](queue, &index); 
+    stop = preprocess_funcs[j](queue, &index, j); 
   if (config.what_to_count & COUNT_CLASS)
     sql_invalidate_shadow_entries(queue, &index);
   idata->ten = index;
@@ -426,14 +444,16 @@ void MY_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
   if (idata->locks == PM_LOCK_EXCLUSIVE) (*sqlfunc_cbr.lock)(bed.p); 
 
   for (idata->current_queue_elem = 0; idata->current_queue_elem < index; idata->current_queue_elem++) {
-    if (queue[idata->current_queue_elem]->valid) sql_query(&bed, queue[idata->current_queue_elem], idata);
+    if (queue[idata->current_queue_elem]->valid)
+      sql_query(&bed, queue[idata->current_queue_elem], idata);
+    if (queue[idata->current_queue_elem]->valid == SQL_CACHE_COMMITTED)
+      LastElemCommitted = queue[idata->current_queue_elem];
   }
 
   /* multi-value INSERT query: wrap-up */
   if (idata->mv.buffer_elem_num) {
     idata->mv.last_queue_elem = TRUE;
-    queue[idata->current_queue_elem-1]->valid = SQL_CACHE_COMMITTED;
-    sql_query(&bed, queue[idata->current_queue_elem-1], idata);
+    sql_query(&bed, LastElemCommitted, idata);
   }
 
   /* rewinding stuff */
@@ -484,11 +504,13 @@ int MY_compose_static_queries()
 {
   int primitives=0, set_primitives=0, have_flows=0;
 
-  if (config.what_to_count & COUNT_FLOWS || (config.sql_table_version >= 4 && !config.sql_optimize_clauses)) {
+  if (config.what_to_count & COUNT_FLOWS || (config.sql_table_version >= 4 &&
+					     config.sql_table_version < SQL_TABLE_VERSION_BGP &&
+					     !config.sql_optimize_clauses)) {
     config.what_to_count |= COUNT_FLOWS;
     have_flows = TRUE;
 
-    if (config.sql_table_version < 4 && !config.sql_optimize_clauses) {
+    if ((config.sql_table_version < 4 || config.sql_table_version >= SQL_TABLE_VERSION_BGP) && !config.sql_optimize_clauses) {
       Log(LOG_ERR, "ERROR ( %s/%s ): The accounting of flows requires SQL table v4. Exiting.\n", config.name, config.type);
       exit_plugin(1);
     }
@@ -643,7 +665,8 @@ void MY_init_default_values(struct insert_data *idata)
   if (!config.sql_db) config.sql_db = mysql_db;
   if (!config.sql_passwd) config.sql_passwd = mysql_pwd;
   if (!config.sql_table) {
-    if (config.sql_table_version == 7) config.sql_table = mysql_table_v7;
+    if (config.sql_table_version == (SQL_TABLE_VERSION_BGP+1)) config.sql_table = mysql_table_bgp;
+    else if (config.sql_table_version == 7) config.sql_table = mysql_table_v7;
     else if (config.sql_table_version == 6) config.sql_table = mysql_table_v6;
     else if (config.sql_table_version == 5) config.sql_table = mysql_table_v5;
     else if (config.sql_table_version == 4) config.sql_table = mysql_table_v4;
