@@ -1,6 +1,22 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2008 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2010 by Paolo Lucente
+*/
+
+/*
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if no, write to the Free Software
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 #include "../include/extract.h"
@@ -145,11 +161,30 @@ struct my_tcphdr
     u_int16_t th_urp;           /* urgent pointer */
 };
 
+struct my_udphdr
+{
+  u_int16_t uh_sport;           /* source port */
+  u_int16_t uh_dport;           /* destination port */
+  u_int16_t uh_ulen;            /* udp length */
+  u_int16_t uh_sum;             /* udp checksum */
+};
+
 struct my_tlhdr {
    u_int16_t	src_port;	/* source and destination ports */
    u_int16_t	dst_port;
 };
 
+struct my_gtphdr {
+    u_int8_t flags;
+    u_int8_t message;
+    u_int16_t length;
+};
+
+/* typedefs */
+typedef u_int32_t as_t;
+typedef u_int16_t as16_t;
+
+
 /* class status */
 struct class_st {
    u_int8_t tentatives;	
@@ -167,12 +202,27 @@ struct packet_ptrs {
   u_char *f_tpl; /* ptr to NetFlow V9 template */
   u_char *f_status; /* ptr to status table entry */
   u_char *idtable; /* ptr to pretag table map */
+  u_char *bpas_table; /* ptr to bgp_peer_as_src table map */
+  u_char *blp_table; /* ptr to bgp_src_local_pref table map */
+  u_char *bmed_table; /* ptr to bgp_src_med table map */
+  u_char *biss_table; /* ptr to bgp_is_symmetric table map */
+  u_char *bta_table; /* ptr to bgp_to_agent table map */
   u_char *packet_ptr; /* ptr to the whole packet */
   u_char *mac_ptr; /* ptr to mac addresses */
   u_int16_t l3_proto; /* layer-3 protocol: IPv4, IPv6 */
   int (*l3_handler)(register struct packet_ptrs *); /* layer-3 protocol handler */
   u_int16_t l4_proto; /* layer-4 protocol */
-  u_int16_t tag; /* pre tag id */
+  pm_id_t tag; /* pre tag id */
+  pm_id_t tag2; /* pre tag id2 */
+  pm_id_t bpas; /* bgp_peer_as_src */
+  pm_id_t blp; /* bgp_src_local_pref */
+  pm_id_t bmed; /* bgp_src_med */
+  pm_id_t biss; /* bgp_is_symmetric */
+  pm_id_t bta; /* bgp_to_agent */
+  char *bgp_src; /* pointer to bgp_node structure for source prefix, if any */  
+  char *bgp_dst; /* pointer to bgp_node structure for destination prefix, if any */ 
+  char *bgp_peer; /* record BGP peer's Router-ID */
+  char *bgp_nexthop; /* record BGP next-hop in case of follow-up */
   u_int16_t pf; /* pending fragments or packets */
   u_int8_t new_flow; /* pmacctd flows: part of a new flow ? */
   u_int8_t tcp_flags; /* pmacctd flows: TCP packet flags; URG, PUSH filtered out */ 
@@ -185,7 +235,10 @@ struct packet_ptrs {
   struct class_st cst; /* classifiers: class status */
   u_int8_t shadow; /* 0=the packet is being distributed for the 1st time
 		      1=the packet is being distributed for the 2nd+ time */
-  u_int8_t tag_dist; /* tagged packet: 0=do not distribute the packet; 1=distribute it */
+  u_int16_t ifindex_in;  /* input ifindex; only used by ULOG for the time being */
+  u_int16_t ifindex_out; /* output ifindex; only used by ULOG for the time being */
+  u_int8_t tun_stack; /* tunnelling stack */
+  u_int8_t tun_layer; /* tunnelling layer count */
 };
 
 struct host_addr {
@@ -206,13 +259,18 @@ struct pkt_primitives {
 #endif
   struct host_addr src_ip;
   struct host_addr dst_ip;
-  u_int16_t src_as;
-  u_int16_t dst_as;
+  u_int8_t src_nmask;
+  u_int8_t dst_nmask;
+  as_t src_as;
+  as_t dst_as;
   u_int16_t src_port;
   u_int16_t dst_port;
   u_int8_t tos;
   u_int8_t proto;
-  u_int16_t id;
+  u_int32_t ifindex_in;
+  u_int32_t ifindex_out;
+  pm_id_t id;
+  pm_id_t id2;
   pm_class_t class;
 };
 
@@ -235,8 +293,15 @@ struct pkt_payload {
   u_int32_t time_start;
   pm_class_t class;
   pm_id_t tag;
+  pm_id_t tag2;
   struct host_addr src_ip;
   struct host_addr dst_ip;
+  as_t src_as;
+  as_t dst_as;
+  u_int16_t ifindex_in;
+  u_int16_t ifindex_out;
+  u_int8_t src_nmask;
+  u_int8_t dst_nmask;
 };
 
 struct pkt_extras {
@@ -244,6 +309,49 @@ struct pkt_extras {
   u_int32_t mpls_top_label;
 };
 
+/* START: BGP section */
+#define MAX_BGP_STD_COMMS       96
+#define MAX_BGP_EXT_COMMS       96
+#define MAX_BGP_ASPATH          128
+
+struct pkt_bgp_primitives {
+  as_t peer_src_as;
+  as_t peer_dst_as;
+  struct host_addr peer_src_ip;
+  struct host_addr peer_dst_ip;
+  char std_comms[MAX_BGP_STD_COMMS];
+  char ext_comms[MAX_BGP_EXT_COMMS];
+  char as_path[MAX_BGP_ASPATH];
+  u_int32_t local_pref;
+  u_int32_t med;
+  char src_std_comms[MAX_BGP_STD_COMMS];
+  char src_ext_comms[MAX_BGP_EXT_COMMS];
+  char src_as_path[MAX_BGP_ASPATH];
+  u_int32_t src_local_pref;
+  u_int32_t src_med;
+  u_int32_t is_symmetric;
+};
+
+/* same as above but pointers in place of strings */
+struct cache_bgp_primitives {
+  as_t peer_src_as;
+  as_t peer_dst_as;
+  struct host_addr peer_src_ip;
+  struct host_addr peer_dst_ip;
+  char *std_comms;
+  char *ext_comms;
+  char *as_path;
+  u_int32_t local_pref;
+  u_int32_t med;
+  char *src_std_comms;
+  char *src_ext_comms;
+  char *src_as_path;
+  u_int32_t src_local_pref;
+  u_int32_t src_med;
+  u_int32_t is_symmetric;
+};
+/* END: BGP section */
+
 struct packet_ptrs_vector {
   struct packet_ptrs v4;
   struct packet_ptrs vlan4;
@@ -258,7 +366,27 @@ struct packet_ptrs_vector {
 };
 
 struct hosts_table {
-  unsigned short int num;
+  short int num;
   struct host_addr table[MAX_MAP_ENTRIES];
 };
 
+#define TUNNEL_PROTO_STRING	16
+#define TUNNEL_REGISTRY_STACKS	9 /* MAX + 1 */
+#define TUNNEL_REGISTRY_ENTRIES 4 
+typedef int (*tunnel_func)(register struct packet_ptrs *);
+
+struct tunnel_handler {
+  tunnel_func tf;
+  u_int8_t proto;
+  u_int16_t port;
+};
+
+typedef int (*tunnel_configurator)(struct tunnel_handler *, char *);
+
+struct tunnel_entry {
+  u_char type[TUNNEL_PROTO_STRING];
+  tunnel_func tf;
+  tunnel_configurator tc;
+};
+
+struct tunnel_handler tunnel_registry[TUNNEL_REGISTRY_STACKS][TUNNEL_REGISTRY_ENTRIES];
