@@ -264,8 +264,8 @@ void handle_hostbyteorder_packet(struct pkt_data *data)
 #endif
   data->primitives.src_port = htons(data->primitives.src_port); 
   data->primitives.dst_port = htons(data->primitives.dst_port); 
-  data->primitives.src_as = htons(data->primitives.src_as); 
-  data->primitives.dst_as = htons(data->primitives.dst_as); 
+  data->primitives.src_as = htonl(data->primitives.src_as); 
+  data->primitives.dst_as = htonl(data->primitives.dst_as); 
 }
 
 /* Fill in transport-layer (tcp/udp) portions of flow record */
@@ -301,15 +301,18 @@ transport_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras
 static int
 l2_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras *extras, int ndx)
 {
-#if defined HAVE_L2
   struct pkt_primitives *p = &data->primitives;
 
+#if defined HAVE_L2
   memcpy(&flow->mac[ndx][0], &p->eth_shost, 6);
   memcpy(&flow->mac[ndx ^ 1][0], &p->eth_dhost, 6);
   flow->vlan = p->vlan_id;
   flow->mpls_label[ndx] = extras->mpls_top_label;
 #endif
 
+  flow->ifindex[ndx] = p->ifindex_in;
+  flow->ifindex[ndx ^ 1] = p->ifindex_out;
+
   return (0);
 }
 
@@ -337,13 +340,16 @@ ipv4_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras *ext
   flow->af = af;
   flow->addr[ndx].v4 = p->src_ip.address.ipv4;
   flow->addr[ndx ^ 1].v4 = p->dst_ip.address.ipv4;
+  flow->mask[ndx] = p->src_nmask;
+  flow->mask[ndx ^ 1] = p->dst_nmask;
   flow->tos[ndx] = p->tos;
   flow->protocol = p->proto;
   flow->octets[ndx] = data->pkt_len;
   flow->packets[ndx] = data->pkt_num;
   flow->flows[ndx] = data->flo_num;
   flow->class = p->class;
-  flow->tag = p->id;
+  flow->tag[ndx] = p->id;
+  flow->tag2[ndx] = p->id2;
 
   *isfrag = 0;
 
@@ -371,11 +377,14 @@ ipv6_to_flowrec(struct FLOW *flow, struct pkt_data *data, struct pkt_extras *ext
   flow->ip6_flowlabel[ndx] = 0;
   flow->addr[ndx].v6 = p->src_ip.address.ipv6; 
   flow->addr[ndx ^ 1].v6 = p->dst_ip.address.ipv6; 
+  flow->mask[ndx] = p->src_nmask;
+  flow->mask[ndx ^ 1] = p->dst_nmask;
   flow->octets[ndx] = data->pkt_len;
   flow->packets[ndx] = data->pkt_num; 
   flow->flows[ndx] = data->flo_num;
   flow->class = p->class;
-  flow->tag = p->id;
+  flow->tag[ndx] = p->id;
+  flow->tag2[ndx] = p->id2;
 
   *isfrag = 0;
 
@@ -553,7 +562,10 @@ process_packet(struct FLOWTRACK *ft, struct pkt_data *data, struct pkt_extras *e
     flow->tcp_flags[1] |= tmp.tcp_flags[1];
     flow->tos[1] = tmp.tos[1]; // XXX
     if (!flow->class) flow->class = tmp.class;
-    if (!flow->tag) flow->tag = tmp.tag;
+    if (!flow->tag[0]) flow->tag[0] = tmp.tag[0];
+    if (!flow->tag[1]) flow->tag[1] = tmp.tag[1];
+    if (!flow->tag2[0]) flow->tag2[0] = tmp.tag2[0];
+    if (!flow->tag2[1]) flow->tag2[1] = tmp.tag2[1];
   }
 	
   memcpy(&flow->flow_last, received_time, sizeof(flow->flow_last));
@@ -1001,23 +1013,36 @@ print_timeouts(struct FLOWTRACK *ft)
 static int
 connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
 {
-	int s;
+	int s, ret = 0;
 	unsigned int h6;
 	unsigned char h4;
 	struct sockaddr_in *in4 = (struct sockaddr_in *)addr;
 	struct sockaddr_in6 *in6 = (struct sockaddr_in6 *)addr;
+	struct host_addr source_ip;
+	struct sockaddr ssource_ip;
+
+	if (config.nfprobe_source_ip) {
+	  ret = str_to_addr(config.nfprobe_source_ip, &source_ip);
+	  addr_to_sa(&ssource_ip, &source_ip, 0);
+	}
 
 	if ((s = socket(addr->ss_family, SOCK_DGRAM, 0)) == -1) {
 		fprintf(stderr, "socket() error: %s\n", 
 		    strerror(errno));
 		exit_plugin(1);
 	}
-	if (connect(s, (struct sockaddr*)addr, len) == -1) {
-		fprintf(stderr, "connect() error: %s\n",
-		    strerror(errno));
-		exit_plugin(1);
+
+	if (ret && bind(s, (struct sockaddr *) &ssource_ip, sizeof(ssource_ip)) == -1) {
+	  fprintf(stderr, "bind() error: %s\n",
+		strerror(errno));
 	}
 
+        if (connect(s, (struct sockaddr*)addr, len) == -1) {
+                fprintf(stderr, "connect() error: %s\n",
+                    strerror(errno));
+                exit_plugin(1);
+        }
+
 	switch (addr->ss_family) {
 	case AF_INET:
 		/* Default to link-local TTL for multicast addresses */
@@ -1392,16 +1417,22 @@ read_data:
       data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
 
       while (((struct ch_buf_hdr *)pipebuf)->num) {
+
 	if (config.networks_file) {
 	  memcpy(&dummy.primitives.src_ip, &data->primitives.src_ip, HostAddrSz);
 	  memcpy(&dummy.primitives.dst_ip, &data->primitives.dst_ip, HostAddrSz);
 
           for (num = 0; net_funcs[num]; num++) (*net_funcs[num])(&nt, &nc, &dummy.primitives);
 
-	  if (((config.acct_type == ACCT_NF || config.acct_type == ACCT_SF) && config.nfacctd_as != NF_AS_KEEP) || config.acct_type == ACCT_PM) {
+	  if (config.nfacctd_as == NF_AS_NEW) {
 	    data->primitives.src_as = dummy.primitives.src_as;
 	    data->primitives.dst_as = dummy.primitives.dst_as;
 	  }
+
+	  if (config.nfacctd_net == NF_NET_NEW) {
+	    data->primitives.src_nmask = dummy.primitives.src_nmask;
+            data->primitives.dst_nmask = dummy.primitives.dst_nmask;
+	  }
 	}
 
 	if (config.ports_file) {
