@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2008 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2010 by Paolo Lucente
 */
 
 /*
@@ -173,7 +173,7 @@ static const struct _protocols_struct _protocols[] = {
   {"", -1},
 };
 
-#ifdef __PMACCTD_C
+#if defined __PMACCTD_C || defined __UACCTD_C
 static struct _devices_struct _devices[] = {
   {null_handler, DLT_NULL},
   {eth_handler, DLT_EN10MB},
@@ -211,11 +211,18 @@ static const struct _dictionary_line dictionary[] = {
   {"aggregate_filter", cfg_key_aggregate_filter},
   {"promisc", cfg_key_promisc},
   {"pcap_filter", cfg_key_pcap_filter},
+  {"pmacctd_as", cfg_key_nfacctd_as_new},
+  {"uacctd_as", cfg_key_nfacctd_as_new},
+  {"pmacctd_net", cfg_key_nfacctd_net},
+  {"uacctd_net", cfg_key_nfacctd_net},
   {"plugins", NULL},
   {"plugin_pipe_size", cfg_key_plugin_pipe_size},
   {"plugin_buffer_size", cfg_key_plugin_buffer_size},
   {"interface", cfg_key_interface},
   {"interface_wait", cfg_key_interface_wait},
+  {"files_umask", cfg_key_files_umask},
+  {"files_uid", cfg_key_files_uid},
+  {"files_gid", cfg_key_files_gid},
   {"savefile_wait", cfg_key_savefile_wait},
   {"networks_mask", cfg_key_networks_mask},
   {"networks_file", cfg_key_networks_file},
@@ -231,6 +238,7 @@ static const struct _dictionary_line dictionary[] = {
   {"sql_table", cfg_key_sql_table},
   {"sql_table_schema", cfg_key_sql_table_schema},
   {"sql_table_version", cfg_key_sql_table_version},
+  {"sql_table_type", cfg_key_sql_table_type},
   {"sql_host", cfg_key_sql_host},
   {"sql_data", cfg_key_sql_data},
   {"sql_backup_host", cfg_key_sql_recovery_backup_host}, /* obsolete */
@@ -264,6 +272,7 @@ static const struct _dictionary_line dictionary[] = {
   {"nfacctd_time_secs", cfg_key_nfacctd_time_secs},
   {"nfacctd_time_new", cfg_key_nfacctd_time_new},
   {"nfacctd_as_new", cfg_key_nfacctd_as_new},
+  {"nfacctd_net", cfg_key_nfacctd_net},
   {"nfacctd_mcast_groups", cfg_key_nfacctd_mcast_groups},
   {"nfacctd_sql_log", cfg_key_nfacctd_sql_log},
   {"pmacctd_force_frag_handling", cfg_key_pmacctd_force_frag_handling},
@@ -272,18 +281,30 @@ static const struct _dictionary_line dictionary[] = {
   {"pmacctd_flow_buffer_buckets", cfg_key_pmacctd_flow_buffer_buckets},
   {"pmacctd_conntrack_buffer_size", cfg_key_pmacctd_conntrack_buffer_size},
   {"pmacctd_flow_lifetime", cfg_key_pmacctd_flow_lifetime},
-  {"pmacctd_id", cfg_key_post_tag},	/* obsolete */
-  {"nfacctd_id_file", cfg_key_pre_tag_map},	/* obsolete */
+  {"pmacctd_ext_sampling_rate", cfg_key_pmacctd_ext_sampling_rate},
+  {"uacctd_force_frag_handling", cfg_key_pmacctd_force_frag_handling},
+  {"uacctd_frag_buffer_size", cfg_key_pmacctd_frag_buffer_size},
+  {"uacctd_flow_buffer_size", cfg_key_pmacctd_flow_buffer_size},
+  {"uacctd_flow_buffer_buckets", cfg_key_pmacctd_flow_buffer_buckets},
+  {"uacctd_conntrack_buffer_size", cfg_key_pmacctd_conntrack_buffer_size},
+  {"uacctd_flow_lifetime", cfg_key_pmacctd_flow_lifetime},
+  {"uacctd_ext_sampling_rate", cfg_key_pmacctd_ext_sampling_rate},
+  {"nfacctd_ext_sampling_rate", cfg_key_pmacctd_ext_sampling_rate},
+  {"sfacctd_ext_sampling_rate", cfg_key_pmacctd_ext_sampling_rate},
   {"pcap_savefile", cfg_key_pcap_savefile},
   {"pre_tag_map", cfg_key_pre_tag_map},	
   {"pre_tag_map_entries", cfg_key_pre_tag_map_entries},	
   {"pre_tag_filter", cfg_key_pre_tag_filter},
+  {"pre_tag2_filter", cfg_key_pre_tag2_filter},
   {"post_tag", cfg_key_post_tag},
   {"sampling_rate", cfg_key_sampling_rate},
   {"sfacctd_port", cfg_key_nfacctd_port},
   {"sfacctd_ip", cfg_key_nfacctd_ip},
   {"sfacctd_allow_file", cfg_key_nfacctd_allow_file},
   {"sfacctd_as_new", cfg_key_nfacctd_as_new},
+  {"sfacctd_net", cfg_key_nfacctd_net},
+  {"uacctd_renormalize", cfg_key_sfacctd_renormalize},
+  {"pmacctd_renormalize", cfg_key_sfacctd_renormalize},
   {"sfacctd_renormalize", cfg_key_sfacctd_renormalize},
   {"nfacctd_renormalize", cfg_key_sfacctd_renormalize},
   {"nfacctd_disable_checks", cfg_key_nfacctd_disable_checks},
@@ -298,13 +319,40 @@ static const struct _dictionary_line dictionary[] = {
   {"nfprobe_receiver", cfg_key_nfprobe_receiver},
   {"nfprobe_engine", cfg_key_nfprobe_engine},
   {"nfprobe_version", cfg_key_nfprobe_version},
+  {"nfprobe_peer_as", cfg_key_nfprobe_peer_as},
+  {"nfprobe_source_ip", cfg_key_nfprobe_source_ip},
   {"sfprobe_receiver", cfg_key_sfprobe_receiver},
-//  {"sfprobe_sampling_rate", cfg_key_sfprobe_sampling_rate},
-  {"sfprobe_sampling_rate", cfg_key_sampling_rate},
   {"sfprobe_agentip", cfg_key_sfprobe_agentip},
   {"sfprobe_agentsubid", cfg_key_sfprobe_agentsubid},
-  {"flow_handling_threads", cfg_key_flow_handling_threads},
-
+  {"sfprobe_peer_as", cfg_key_nfprobe_peer_as},
+  {"bgp_daemon", cfg_key_nfacctd_bgp},
+  {"bgp_daemon_ip", cfg_key_nfacctd_bgp_ip},
+  {"bgp_daemon_port", cfg_key_nfacctd_bgp_port},
+  {"bgp_daemon_max_peers", cfg_key_nfacctd_bgp_max_peers},
+  {"bgp_daemon_msglog", cfg_key_nfacctd_bgp_msglog},
+  {"bgp_daemon_allow_file", cfg_key_nfacctd_bgp_allow_file},
+  {"bgp_aspath_radius", cfg_key_nfacctd_bgp_aspath_radius},
+  {"bgp_stdcomm_pattern", cfg_key_nfacctd_bgp_stdcomm_pattern},
+  {"bgp_extcomm_pattern", cfg_key_nfacctd_bgp_extcomm_pattern},
+  {"bgp_stdcomm_pattern_to_asn", cfg_key_nfacctd_bgp_stdcomm_pattern_to_asn},
+  {"bgp_peer_as_skip_subas", cfg_key_nfacctd_bgp_peer_as_skip_subas},
+  {"bgp_peer_src_as_map", cfg_key_nfacctd_bgp_peer_src_as_map},
+  {"bgp_src_local_pref_map", cfg_key_nfacctd_bgp_src_local_pref_map},
+  {"bgp_src_med_map", cfg_key_nfacctd_bgp_src_med_map},
+  {"bgp_is_symmetric_map", cfg_key_nfacctd_bgp_is_symmetric_map},
+  {"bgp_peer_src_as_type", cfg_key_nfacctd_bgp_peer_src_as_type},
+  {"bgp_src_std_comm_type", cfg_key_nfacctd_bgp_src_std_comm_type},
+  {"bgp_src_ext_comm_type", cfg_key_nfacctd_bgp_src_ext_comm_type},
+  {"bgp_src_as_path_type", cfg_key_nfacctd_bgp_src_as_path_type},
+  {"bgp_src_local_pref_type", cfg_key_nfacctd_bgp_src_local_pref_type},
+  {"bgp_src_med_type", cfg_key_nfacctd_bgp_src_med_type},
+  {"bgp_agent_map", cfg_key_nfacctd_bgp_to_agent_map},
+  {"bgp_follow_default", cfg_key_nfacctd_bgp_follow_default},
+  {"bgp_follow_nexthop", cfg_key_nfacctd_bgp_follow_nexthop},
+  {"bgp_neighbors_file", cfg_key_nfacctd_bgp_neighbors_file},
+  {"uacctd_group", cfg_key_uacctd_group},
+  {"uacctd_nl_size", cfg_key_uacctd_nl_size},
+  {"tunnel_0", cfg_key_tunnel_0},
   {"", NULL},
 };
 
@@ -325,5 +373,12 @@ static struct plugin_type_entry plugin_types_list[] = {
 #endif
   {PLUGIN_ID_UNKNOWN,	"",		NULL},
 };
+#endif
+
+#ifdef __NL_C
+static struct tunnel_entry tunnel_handlers_list[] = {
+  {"gtp", 	gtp_tunnel_func, 	gtp_tunnel_configurator},
+  {"", 		NULL,			NULL},
+};
 
 #endif
