@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2008 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2010 by Paolo Lucente
 */
 
 /*
@@ -192,8 +192,16 @@ struct pcap_device {
 };
 
 struct pcap_callback_data {
+  u_char * f_agent; 
   u_char * idt; 
+  u_char * bta_table;
+  u_char * bpas_table; 
+  u_char * blp_table; 
+  u_char * bmed_table; 
+  u_char * biss_table; 
   struct pcap_device *device;
+  u_int16_t ifindex_in;
+  u_int16_t ifindex_out;
 };
 
 struct _protocols_struct {
@@ -258,19 +266,22 @@ EXT u_char *llc_handler(const struct pcap_pkthdr *, u_int, register u_char *, re
 EXT void chdlc_handler(const struct pcap_pkthdr *, register struct packet_ptrs *);
 #undef EXT
 
-#if (!defined __PMACCTD_C) 
+#if (!defined __NL_C)
 #define EXT extern
 #else
 #define EXT
 #endif
 EXT int ip_handler(register struct packet_ptrs *);
 EXT int ip6_handler(register struct packet_ptrs *);
+EXT int gtp_tunnel_func(register struct packet_ptrs *);
+EXT int gtp_tunnel_configurator(struct tunnel_handler *, char *);
+EXT void tunnel_registry_init();
 EXT void pcap_cb(u_char *, const struct pcap_pkthdr *, const u_char *);
-EXT int PM_find_id(struct packet_ptrs *);
+EXT void PM_find_id(struct id_table *, struct packet_ptrs *, pm_id_t *, pm_id_t *);
 EXT void compute_once();
 #undef EXT
 
-#if (!defined __PMACCTD_C) && (!defined __NFACCTD_C) && (!defined __SFACCTD_C)
+#if (!defined __PMACCTD_C) && (!defined __NFACCTD_C) && (!defined __SFACCTD_C) && (!defined __UACCTD_C)
 #define EXT extern
 #else
 #define EXT
@@ -286,7 +297,7 @@ size_t strlcpy(char *, const char *, size_t);
 pcap_t *glob_pcapt;
 struct pcap_stat ps;
 
-#if (!defined __PMACCTD_C) && (!defined __NFACCTD_C) && (!defined __SFACCTD_C)
+#if (!defined __PMACCTD_C) && (!defined __NFACCTD_C) && (!defined __SFACCTD_C) && (!defined __UACCTD_C)
 extern int debug;
 extern int have_num_memory_pools; /* global getopt() stuff */
 extern struct configuration config; /* global configuration structure */ 
