@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2008 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2010 by Paolo Lucente
 */
 
 /*
@@ -19,8 +19,9 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-static const struct _map_dictionary_line map_dictionary[] = {
+const struct _map_dictionary_line tag_map_dictionary[] = {
   {"id", PT_map_id_handler},
+  {"id2", PT_map_id2_handler},
   {"ip", PT_map_ip_handler},
   {"in", PT_map_input_handler},
   {"out", PT_map_output_handler},
@@ -32,11 +33,34 @@ static const struct _map_dictionary_line map_dictionary[] = {
   {"v8agg", PT_map_v8agg_handler},
   {"agent_id", PT_map_agent_id_handler},
   {"sampling_rate", PT_map_sampling_rate_handler},
+  {"direction", PT_map_direction_handler},
   {"src_as", PT_map_src_as_handler},
   {"dst_as", PT_map_dst_as_handler},
+  {"peer_src_as", PT_map_peer_src_as_handler},
+  {"peer_dst_as", PT_map_peer_dst_as_handler},
+  {"src_local_pref", PT_map_src_local_pref_handler},
+  {"local_pref", PT_map_local_pref_handler},
+  {"src_comms", PT_map_src_comms_handler},
+  {"comms", PT_map_comms_handler},
   {"label", PT_map_label_handler},
   {"jeq", PT_map_jeq_handler},
   {"return", PT_map_return_handler},
   {"stack", PT_map_stack_handler},
   {"", NULL}
 };
+
+const struct _map_dictionary_line bpas_map_dictionary[] = {
+  {"id", PT_map_id_handler},
+  {"ip", PT_map_ip_handler},
+  {"in", PT_map_input_handler},
+  {"bgp_nexthop", BPAS_map_bgp_nexthop_handler},
+  {"peer_dst_as", BPAS_map_bgp_peer_dst_as_handler},
+  {"src_mac", BPAS_map_src_mac_handler},
+  {"", NULL}
+};
+
+const struct _map_dictionary_line bta_map_dictionary[] = {
+  {"id", PT_map_id_handler},
+  {"ip", PT_map_ip_handler},
+  {"", NULL}
+};
